Package: TRONCO
Version: 2.43.0
Date: 2025-09-10
Title: TRONCO, an R package for TRanslational ONCOlogy
Authors@R: c(person("Marco", "Antoniotti", role=c("ctb")), 
    person("Giulio", "Caravagna", role=c("aut"), email="gcaravagna@units.it"),
    person("Luca", "De Sano", role=c("cre","aut"), email="luca.desano@gmail.com",
                comment = c(ORCID = "0000-0002-9618-3774")), 
    person("Alex", "Graudenzi", role=c("aut"), email="alex.graudenzi@unimib.it"),
    person("Giancarlo", "Mauri", role=c("ctb")), 
    person("Bud", "Mishra", role=c("ctb")), 
    person("Daniele", "Ramazzotti", role=c("aut"),email="daniele.ramazzotti@unimib.it",
                comment = c(ORCID = "0000-0002-6087-2666")))
Depends: R (>= 4.1.0),
Imports: bnlearn, Rgraphviz, gtools, parallel, foreach, doParallel,
        iterators, RColorBrewer, circlize, igraph, grid, gridExtra,
        xtable, gtable, scales, R.matlab, grDevices, graphics, stats,
        utils, methods
Suggests: BiocGenerics, BiocStyle, testthat, knitr, rWikiPathways,
        magick
Name: An R package for the inference of cancer progression models from
        heterogeneous genomic data
Description: 
    The TRONCO (TRanslational ONCOlogy) R package collects algorithms to infer
    progression models via the approach of Suppes-Bayes Causal Network, both 
    from an ensemble of tumors (cross-sectional samples) and within an individual 
    patient (multi-region or single-cell samples). The package provides parallel 
    implementation of algorithms that process binary matrices where each row 
    represents a tumor sample and each column a single-nucleotide or a structural 
    variant driving the  progression; a 0/1 value models the absence/presence of 
    that alteration in the sample. The tool can import data from plain, MAF or 
    GISTIC format files, and can fetch it from the cBioPortal for cancer genomics. 
    Functions for  data manipulation and visualization are provided, as well as 
    functions to import/export such data to other bioinformatics  tools for, e.g, 
    clustering or detection of mutually exclusive alterations. Inferred models can be 
    visualized and tested for their confidence via bootstrap and cross-validation.
    TRONCO is used for  the implementation of the Pipeline for Cancer Inference (PICNIC). 
Encoding: UTF-8
License: GPL-3
URL: https://sites.google.com/site/troncopackage/
BugReports: https://github.com/BIMIB-DISCo/TRONCO
biocViews: BiomedicalInformatics, Bayesian, GraphAndNetwork,
        SomaticMutation, NetworkInference, Network, Clustering,
        DataImport, SingleCell, ImmunoOncology
RoxygenNote: 7.3.3
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/TRONCO
git_branch: devel
git_last_commit: 7420d7e
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-30
NeedsCompilation: no
Packaged: 2025-10-30 23:31:21 UTC; biocbuild
Author: Marco Antoniotti [ctb],
  Giulio Caravagna [aut],
  Luca De Sano [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-9618-3774>),
  Alex Graudenzi [aut],
  Giancarlo Mauri [ctb],
  Bud Mishra [ctb],
  Daniele Ramazzotti [aut] (ORCID:
    <https://orcid.org/0000-0002-6087-2666>)
Maintainer: Luca De Sano <luca.desano@gmail.com>
