# Generated by roxygen2: do not edit by hand

export(AND)
export(OR)
export(TCGA.map.clinical.data)
export(TCGA.multiple.samples)
export(TCGA.remove.multiple.samples)
export(TCGA.shorten.barcodes)
export(XOR)
export(annotate.description)
export(annotate.stages)
export(as.adj.matrix)
export(as.alterations)
export(as.bootstrap.scores)
export(as.colors)
export(as.conditional.probs)
export(as.confidence)
export(as.description)
export(as.events)
export(as.events.in.patterns)
export(as.events.in.sample)
export(as.gene)
export(as.genes)
export(as.genes.in.patterns)
export(as.genotypes)
export(as.hypotheses)
export(as.joint.probs)
export(as.kfold.eloss)
export(as.kfold.posterr)
export(as.kfold.prederr)
export(as.marginal.probs)
export(as.models)
export(as.parameters)
export(as.pathway)
export(as.patterns)
export(as.samples)
export(as.selective.advantage.relations)
export(as.stages)
export(as.types)
export(as.types.in.patterns)
export(change.color)
export(consolidate.data)
export(delete.event)
export(delete.gene)
export(delete.hypothesis)
export(delete.model)
export(delete.pattern)
export(delete.samples)
export(delete.type)
export(duplicates)
export(ebind)
export(enforce.numeric)
export(enforce.string)
export(events.selection)
export(export.graphml)
export(export.mutex)
export(export.nbs.input)
export(extract.MAF.HuGO.Entrez.map)
export(genes.table.report)
export(has.duplicates)
export(has.model)
export(has.stages)
export(hypothesis.add)
export(hypothesis.add.group)
export(hypothesis.add.homologous)
export(import.GISTIC)
export(import.MAF)
export(import.genotypes)
export(import.model)
export(import.mutex.groups)
export(intersect.datasets)
export(is.compliant)
export(join.events)
export(join.types)
export(keysToNames)
export(nameToKey)
export(nevents)
export(ngenes)
export(nhypotheses)
export(npatterns)
export(nsamples)
export(ntypes)
export(oncoprint)
export(oncoprint.cbio)
export(order.frequency)
export(pathway.visualization)
export(pheatmap)
export(rank.recurrents)
export(rename.gene)
export(rename.type)
export(samples.selection)
export(sbind)
export(ssplit)
export(trim)
export(tronco.bootstrap)
export(tronco.caprese)
export(tronco.capri)
export(tronco.chowliu)
export(tronco.edmonds)
export(tronco.gabow)
export(tronco.kfold.eloss)
export(tronco.kfold.posterr)
export(tronco.kfold.prederr)
export(tronco.pattern.plot)
export(tronco.plot)
export(tronco.prim)
export(view)
export(which.samples)
importFrom(R.matlab,writeMat)
importFrom(RColorBrewer,brewer.pal)
importFrom(RColorBrewer,brewer.pal.info)
importFrom(Rgraphviz,edgeNames)
importFrom(Rgraphviz,plot)
importFrom(bnlearn,"amat<-")
importFrom(bnlearn,amat)
importFrom(bnlearn,bn.cv)
importFrom(bnlearn,chow.liu)
importFrom(bnlearn,empty.graph)
importFrom(bnlearn,hc)
importFrom(bnlearn,score)
importFrom(bnlearn,set.arc)
importFrom(bnlearn,tabu)
importFrom(circlize,chordDiagram)
importFrom(circlize,circos.clear)
importFrom(circlize,circos.par)
importFrom(circlize,circos.text)
importFrom(circlize,circos.trackPlotRegion)
importFrom(circlize,get.cell.meta.data)
importFrom(doParallel,registerDoParallel)
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(grDevices,bmp)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.copy2pdf)
importFrom(grDevices,dev.cur)
importFrom(grDevices,dev.off)
importFrom(grDevices,dev.set)
importFrom(grDevices,jpeg)
importFrom(grDevices,pdf)
importFrom(grDevices,png)
importFrom(grDevices,rainbow)
importFrom(grDevices,rgb)
importFrom(grDevices,tiff)
importFrom(graphics,barplot)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,locator)
importFrom(graphics,par)
importFrom(graphics,plot.new)
importFrom(graphics,strwidth)
importFrom(grid,convertHeight)
importFrom(grid,convertWidth)
importFrom(grid,gList)
importFrom(grid,gTree)
importFrom(grid,gpar)
importFrom(grid,grid.draw)
importFrom(grid,grid.newpage)
importFrom(grid,grid.pretty)
importFrom(grid,grobTree)
importFrom(grid,polylineGrob)
importFrom(grid,rectGrob)
importFrom(grid,textGrob)
importFrom(grid,unit)
importFrom(grid,unit.c)
importFrom(grid,viewport)
importFrom(gridExtra,grid.arrange)
importFrom(gridExtra,grid.table)
importFrom(gtable,gtable)
importFrom(gtable,gtable_add_grob)
importFrom(gtable,gtable_height)
importFrom(gtable,gtable_width)
importFrom(gtools,permutations)
importFrom(igraph,"E<-")
importFrom(igraph,"V<-")
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,clusters)
importFrom(igraph,edge)
importFrom(igraph,get.adjacency)
importFrom(igraph,get.edgelist)
importFrom(igraph,get.shortest.paths)
importFrom(igraph,graph.adjacency)
importFrom(igraph,graph.union)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,igraph.from.graphNEL)
importFrom(igraph,igraph.to.graphNEL)
importFrom(igraph,is.dag)
importFrom(igraph,minimum.spanning.tree)
importFrom(igraph,set.edge.attribute)
importFrom(igraph,set.graph.attribute)
importFrom(igraph,set.vertex.attribute)
importFrom(igraph,unfold.tree)
importFrom(igraph,write.graph)
importFrom(iterators,icount)
importFrom(methods,getPackageName)
importFrom(methods,is)
importFrom(parallel,detectCores)
importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
importFrom(scales,brewer_pal)
importFrom(scales,dscale)
importFrom(scales,hue_pal)
importFrom(stats,AIC)
importFrom(stats,BIC)
importFrom(stats,as.dist)
importFrom(stats,cor)
importFrom(stats,cutree)
importFrom(stats,dist)
importFrom(stats,fisher.test)
importFrom(stats,hclust)
importFrom(stats,kmeans)
importFrom(stats,logLik)
importFrom(stats,phyper)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,wilcox.test)
importFrom(utils,combn)
importFrom(utils,count.fields)
importFrom(utils,flush.console)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,read.delim)
importFrom(utils,read.table)
importFrom(utils,write.table)
importFrom(xtable,xtable)
