## ----include=FALSE------------------------------------------------------------
library(TRONCO)
data(aCML)
data(crc_maf)
data(crc_gistic)
data(crc_plain)

## ----fig.show='hide', warning=FALSE, eval=FALSE-------------------------------
# export.graphml(model.boot,
#     file = 'graph.gml',
#     fontsize = 12,
#     scale.nodes = .6,
#     height.logic = 0.25)

