% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{as.parameters}
\alias{as.parameters}
\title{as.parameters}
\usage{
as.parameters(x)
}
\arguments{
\item{x}{A TRONCO model.}
}
\value{
A list of parameters
}
\description{
Get parameters of a model
}
\examples{
data(test_model)
as.parameters(test_model)

}
