% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{as.events.in.patterns}
\alias{as.events.in.patterns}
\title{as.events.in.patterns}
\usage{
as.events.in.patterns(x, patterns = NULL)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{patterns}{A list of patterns for which the list will be returned}
}
\value{
A list of events present in patterns which consitute CAPRI's hypotheses
}
\description{
Return the list of events present in selected patterns
}
\examples{
data(test_dataset)
as.events.in.patterns(test_dataset)
as.events.in.patterns(test_dataset, patterns='XOR_EZH2')

}
