% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{pathway.visualization}
\alias{pathway.visualization}
\title{pathway.visualization}
\usage{
pathway.visualization(
  x,
  title = paste("Pathways:", paste(names(pathways), collapse = ", ", sep = "")),
  file = NA,
  pathways.color = "Set2",
  aggregate.pathways,
  pathways,
  ...
)
}
\arguments{
\item{x}{A TRONCO complian dataset}

\item{title}{Plot title}

\item{file}{To generate a PDF a filename have to be given}

\item{pathways.color}{A RColorBrewer color palette}

\item{aggregate.pathways}{Boolean parameter}

\item{pathways}{Pathways}

\item{...}{Additional parameters}
}
\value{
plot information
}
\description{
Visualise pathways informations
}
