% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{test_model_kfold}
\alias{test_model_kfold}
\title{A complete dataset with a reconstructed model and crossvalidation informations}
\format{
TRONCO compliant dataset
}
\source{
fake data
}
\usage{
data(test_model_kfold)
}
\value{
A standard TRONCO object
}
\description{
This dataset contains a model reconstructed with CAPRI
}
\author{
Luca De Sano
}
