% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{as.kfold.eloss}
\alias{as.kfold.eloss}
\title{as.kfold.eloss}
\usage{
as.kfold.eloss(x, models = names(x$model), values = FALSE)
}
\arguments{
\item{x}{A TRONCO model.}

\item{models}{A subset of reconstructed models, all by default.}

\item{values}{If you want to see also the values}
}
\value{
All the bootstrap scores in a TRONCO model
}
\description{
Returns a dataframe with all the average/stdev entropy loss score of a 
TRONCO model. It is possible to specify models if multiple
reconstruction have been performed.
}
\examples{
data(test_model_kfold)
as.kfold.eloss(test_model_kfold)
as.kfold.eloss(test_model_kfold, models='capri_aic')

}
